SystemOrganization addCategory: #GODGraphGenerator!

Morph subclass: #GGGraph
	instanceVariableNames: 'horizontalLabel verticalLabel dataTable'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODGraphGenerator'!

GGGraph subclass: #GGBarGraph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODGraphGenerator'!

!GGBarGraph methodsFor: 'drawing' stamp: 'WS 11/3/2014 01:01'!
drawBar: canvas at: origin width: width height: height label: label
	"Draws a bar into the canvas at the given origin, with the given dimensions, named with a label. The drawn bar is blue."

	| r |
	r := Rectangle new.
	r setOrigin: ( canvas origin + ( ( origin x ) @ ( origin y - height ) ) ) corner: ( canvas origin + ( ( origin x + width ) @ ( origin y ) ) ).
	canvas fillRectangle: r color: Color blue.
	label asMorph position: ( ( origin x + ( width // 2 ) ) - ( 7 * label size // 2 ) ) @ ( origin y - height - 14 ); drawOn: canvas.! !

!GGBarGraph methodsFor: 'drawing' stamp: 'WS 11/3/2014 01:01'!
drawOn: canvas
	"Draws the bar graph in the canvas."

	| i |
	self drawAxis: canvas.
	i := 0.
	dataTable keysDo: [ :key |
		self drawBar: canvas at: ( 14 + ( i * 40 ) + ( ( i + 1 ) * 10 ) ) @ ( self height - 14 ) width: 40 height: ( self dataTable at: key ) label: key.
		i := i + 1.
	]! !

!GGGraph methodsFor: 'accessing' stamp: 'WS 11/3/2014 00:57'!
dataTable
	"Gets the table containing the data to be plotted on the graph."

	^ dataTable! !

!GGGraph methodsFor: 'drawing' stamp: 'WS 11/5/2014 10:20'!
drawAxis: canvas
	"Draws the axis of the graph and its labels."

	| v origin |
	canvas fillColor: Color white.
	v := TransformMorph new addMorph: self verticalLabel asMorph; angle: 1.57; offset: 0 @ ( -7 * self verticalLabel size ); color: Color white.
	self addMorph: v.
	v extent: 14 @ ( 7.25 * self verticalLabel size ).
	canvas drawString: self horizontalLabel at: ( self width - ( 7 * self horizontalLabel size ) ) @ ( self height - 14 ).
	origin := self graphOriginFromAxis.
	canvas line: origin x @ 0 to: origin color: Color black.
	canvas line: origin to: self width @ origin y color: Color black.! !

!GGGraph methodsFor: 'drawing' stamp: 'WS 11/4/2014 23:17'!
drawOn: canvas
	"Draws the graph on a canvas. Subclasses must implement."

	^ self subclassResponsibility.! !

!GGGraph methodsFor: 'export' stamp: 'WS 11/5/2014 09:06'!
exportToFile: filename
	"Export the graph to a PNG file. The file is located at {#squeak_dir}/Contents/Resources"

	| dir file |
	file := filename, '.png'.
	dir := FileDirectory forFileName: file.
	dir deleteFileNamed: file.
	self exportAsPNGNamed: file! !

!GGGraph methodsFor: 'generate' stamp: 'WS 11/3/2014 00:58'!
generateGraph: aDictionary
	"Sets the dataTable and generate the graph Morph. Makes this instance draw the graph represented by the data when drawn into some canvas."

	dataTable := aDictionary.
	dataTable isDictionary ifFalse: GGDictionaryMustBeADictionary.
	dataTable isEmpty ifTrue: GGDictionaryMustNotBeEmpty.
	self width: ( 7 * self horizontalLabel size + 14 ) + ( dataTable size * 60 ) height: dataTable values asOrderedCollection max + ( 7 * self verticalLabel size + 14 )! !

!GGGraph methodsFor: 'generate' stamp: 'WS 11/3/2014 00:59'!
generateGraph: dictionary toFile: filename
	"Generate the graph and export it to a png file. The file is located at {#squeak_dir}/Contents/Resources."

	self generateGraph: dictionary.
	self exportToFile: filename.! !

!GGGraph methodsFor: 'private' stamp: 'WS 11/2/2014 23:36'!
graphOriginFromAxis
	"Gets the origin point of the graph, considering the axis"

	^ 14 @ ( self height - 14 )! !

!GGGraph methodsFor: 'initialize-release' stamp: 'WS 11/5/2014 09:17'!
height: h
	"Defines the height of this graph. Must be greater than 14."

	h > 14 ifFalse: [ ^ GGHeightTooSmall signal ].
	super height: h! !

!GGGraph methodsFor: 'accessing' stamp: 'pm 10/5/2014 21:46'!
horizontalLabel
	^ horizontalLabel! !

!GGGraph methodsFor: 'accessing' stamp: 'WS 11/12/2014 00:56'!
horizontalLabel: aString
	"Defines a label to the horizontal axis. Accepts a string or nil (meaning no label)."

	| label |
	aString = nil ifTrue: [ label := '' ] ifFalse: [ label := aString ].
	label isString ifFalse: [ self error: 'String expected' ].
	horizontalLabel := label.
	^self! !

!GGGraph methodsFor: 'accessing' stamp: 'pm 10/5/2014 21:46'!
verticalLabel
	^ verticalLabel! !

!GGGraph methodsFor: 'accessing' stamp: 'WS 11/12/2014 00:56'!
verticalLabel: aString
	"Defines a label to the vertical axis. Accepts a string or nil (meaning no label)."

	| label |
	aString = nil ifTrue: [ label := '' ] ifFalse: [ label := aString ].
	label isString ifFalse: [ self error: 'String expected' ].
	verticalLabel := label.
	^self! !

!GGGraph methodsFor: 'initialize-release' stamp: 'WS 11/5/2014 09:20'!
width: w
	"Defines the width of this graph. Must be greater than 14."

	w > 14 ifFalse: [ ^ GGWidthTooSmall signal ].
	super width: w! !

!GGGraph methodsFor: 'initialize-release' stamp: 'WS 11/5/2014 01:41'!
width: w height: h
	"Defines the width and height of the graph"

	self width: w.
	self height: h.
	^self! !

GGGraph subclass: #GGLineGraph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODGraphGenerator'!

!GGLineGraph methodsFor: 'drawing' stamp: 'WS 11/2/2014 17:29'!
drawOn: canvas
	"Draws this Line Graph into a Canvas"

	| i lastPoint origin |
	self drawAxis: canvas.
	i := 0.
	origin := self graphOriginFromAxis.
	lastPoint := nil.
	dataTable associationsDo: [ :entry |
		lastPoint = nil ifFalse: [
		canvas line: lastPoint to: origin + ( entry key @ ( ( entry value ) negated ) ) color: Color blue.
		lastPoint := origin + ( entry key @ ( ( entry value ) negated ) ).
		] ifTrue: [ lastPoint := origin + ( entry key @ ( ( entry value ) negated ) ). ]
	]! !
